@@ -11,8 +11,9 @@ use std::{
 };
 
 use crate::{
-    build::CircuitBuilder, mgr::AbstractCircBuilder, timeouts, AbstractCirc, CircMgr, CircMgrInner,
-    Error, Result,
+    build::{onion_circparams_from_netparams, CircuitBuilder},
+    mgr::AbstractCircBuilder,
+    timeouts, AbstractCirc, CircMgr, CircMgrInner, Error, Result,
 };
 use futures::{task::SpawnExt, StreamExt, TryFutureExt};
 use once_cell::sync::OnceCell;
@@ -431,7 +432,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
             .into());
         }
 
-        let params = crate::DirInfo::from(netdir).circ_params();
+        let params = onion_circparams_from_netparams(netdir.params())?;
         self.extend_circ(circ, params, target).await
     }
 
@@ -638,7 +639,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
         match (circuit.kind, kind) {
             (HsCircStemKind::Naive, HsCircStemKind::Guarded) => {
                 debug!("Wanted GUARDED circuit, but got NAIVE; extending by 1 hop...");
-                let params = CircParameters::default();
+                let params = crate::build::onion_circparams_from_netparams(netdir.params())?;
                 let circ_path = circuit.circ.path_ref();
 
                 // A NAIVE circuit is a 3-hop circuit.
